/* FILE: Buckets.H   last change: Feb 1998
 */

#ifndef _buckets_h
#define _buckets_h

#include <iostream.h>
#include <List.H>
#include <limits.h>

// nodes 1, ... ,num_nodes

// buckets[value][...]  is an array containing
// size[value]  nodes with a given "value".
// pos[node] is the pos of a given node in a given bucket.
// value >= min_value  and  value <= max_value,
// where min_value = - max_value. 

// note: insertion is done at the end of each bucket.

   
class Buckets
 {
   int  max_value;  //is set up at creation as a parameter of the constructor.
     //  an instance of "Buckets" has the following  2*max_value +1  buckets:
     //   buckets[-max_value], ... , buckets[max_value -1], buckets[max_value]

   List* *bucket; 
     // *bucket[value]  is a list containing nodes with a given "value".

   int  max, min; 
       // max and min  values of the nodes actually present into the buckets.

   ListPos *pos;   // position of a node into his list.


 public:
    Buckets(int num_nodes, int my_max_value);
    ~Buckets();

    void insertFirst(int node, int value);
    void insertLast(int node, int value);

    void upFirst(int node, int value);
    void downFirst(int node, int value);
    void upLast(int node, int value);
    void downLast(int node, int value);

    void moveFirst(int node, int value, int delta);
    void moveLast(int node, int value, int delta);

    int extract_first_max_node();
    int extract_first_min_node();
    int extract_last_min_node();

    void del(int node, int value);

    int max_node_allowed(const int *tabu);
    int Max()  { return  max; }
    int Min()  { return  min; }

    int first_max_node();
};



inline void Buckets::insertFirst(int node, int value)  {
   if(value > max)  max = value;
   else if(value < min)  min = value;
     
   pos[node] = bucket[value]->InsertFirst(node);
}

inline void Buckets::insertLast(int node, int value)  {
   if(value > max)  max = value;
   else if(value < min)  min = value;
     
   pos[node] = bucket[value]->InsertLast(node);
}



inline int Buckets::max_node_allowed(const int *tabu)  {
  int pippo;
  for(int value = max; ; value--)  {
     int count = bucket[value]->Size();
     bucket[value]->GoToFirst();
     while(count--)
     {
        pippo = bucket[value]->Read();
        if(!tabu[pippo])  return  pippo;
        bucket[value]->Next();
     }

  }
}


inline int Buckets::extract_first_max_node()  {
   bucket[max]->GoToFirst();
   int node = bucket[max]->Read();
   bucket[max]->Delete();
     
   // max  must be updated!
   while(bucket[max]->IsEmpty() && max > min)  max--;

   return  node;
} 


inline int Buckets::extract_first_min_node()  {
   bucket[min]->GoToFirst();
   int node = bucket[min]->Read();
   bucket[min]->Delete();
     
   // min  must be updated!
   while(bucket[min]->IsEmpty() && min < max)  min++;

   return  node;
} 


inline int Buckets::extract_last_min_node()  {
   bucket[min]->GoToLast();
   int node = bucket[min]->Read();
   bucket[min]->Delete();
     
   // min  must be updated!
   while(bucket[min]->IsEmpty() && min < max)  min++;

   return  node;
} 



inline int Buckets::first_max_node()  {
   bucket[max]->GoToFirst();
   return  bucket[max]->Read();
} 





inline void Buckets::upFirst(int node, int value)  { // the value of node is increased by 1.

   // first, remove node from the list *bucket[value].
    bucket[value]->GoTo(pos[node]);
    bucket[value]->Delete();
     
   // then, insert node back with the new value.
    value++;
    pos[node] = bucket[value]->InsertFirst(node);

   // max  and  min  must be updated!
    if(bucket[min]->IsEmpty())  min++;
    if(value > max)  max++;
}


inline void Buckets::upLast(int node, int value)  { // the value of node is increased by 1.

   // first, remove node from the list *bucket[value].
    bucket[value]->GoTo(pos[node]);
    bucket[value]->Delete();
     
   // then, insert node back with the new value.
    value++;
    pos[node] = bucket[value]->InsertLast(node);

   // max  and  min  must be updated!
    if(bucket[min]->IsEmpty())  min++;
    if(value > max)  max++;
}


inline void Buckets::downFirst(int node, int value)  { // the value of node is decreased by 1.

   // first, remove node from the list *bucket[value].
    bucket[value]->GoTo(pos[node]);
    bucket[value]->Delete();
     
   // then, insert node back with the new value.
    value--;
    pos[node] = bucket[value]->InsertFirst(node);

   // max  and  min  must be updated!
    if(bucket[max]->IsEmpty())  max--;
    if(value < min)  min--;
}


inline void Buckets::downLast(int node, int value)  { // the value of node is decreased by 1.

   // first, remove node from the list *bucket[value].
    bucket[value]->GoTo(pos[node]);
    bucket[value]->Delete();
     
   // then, insert node back with the new value.
    value--;
    pos[node] = bucket[value]->InsertLast(node);

   // max  and  min  must be updated!
    if(bucket[max]->IsEmpty())  max--;
    if(value < min)  min--;
}


inline void Buckets::moveFirst(int node, int value, int delta)  {

   // first, remove node from the list *bucket[value].
    bucket[value]->GoTo(pos[node]);
    bucket[value]->Delete();
     
   // then, insert node back with the new value.
    value += delta;
    pos[node] = bucket[value]->InsertFirst(node);

   // max  and  min  must be updated!
    if(value > max)  max = value;
    if(value < min)  min = value;
    while(bucket[min]->IsEmpty())  min++;
    while(bucket[max]->IsEmpty())  max--;
}


inline void Buckets::moveLast(int node, int value, int delta)  {

   // first, remove node from the list *bucket[value].
    bucket[value]->GoTo(pos[node]);
    bucket[value]->Delete();
     
   // then, insert node back with the new value.
    value += delta;
    pos[node] = bucket[value]->InsertLast(node);

   // max  and  min  must be updated!
    if(value > max)  max = value;
    if(value < min)  min = value;
    while(bucket[min]->IsEmpty())  min++;
    while(bucket[max]->IsEmpty())  max--;
}




inline void Buckets::del( int node, int value)  {

   bucket[value]->GoTo(pos[node]);
   bucket[value]->Delete();


  // max and min must be updated!
  if(bucket[value]->IsEmpty()) {
     if(value == max)  {
        if(max != min)  {
           max--;  
           while(bucket[max]->IsEmpty())  max--;
        }
        else { max = INT_MIN;  min = INT_MAX; }
     }  
     else if(value == min)  {
        min++;  
        while(bucket[min]->IsEmpty())  min++;  
     }
  }
} 





#endif











