//
// Classe per rappresentare una Lista. Doppi puntatori con "sentinella"
//



#ifndef LIST_HDR
#define LIST_HDR




typedef int mytype;		// Il tipo dei nostri dati


class Node {
  friend class List;		// List puo` accedere alla parte privata

  Node *prev;			// Puntatore all'elemento precedente
  Node *next;			// Puntatore all'elemento successivo
  mytype data;			// L'elemento memorizzato nel nodo
};



typedef Node* ListPos;		// La posizione nella Lista e` un puntatore
				// ad un nodo



class List {
  Node *sentinel;		// Puntatore alla Sentinella
  Node *current;		// Puntatore all'elemento corrente
  int len;			// La lunghezza della Lista

public:
  List();			// Costruttore
  ~List();			// Distruttore

  // Spostamenti sulla lista
  ListPos Next();		// Sposta in avanti
  ListPos Previous();		// Sposta indietro
  ListPos GoTo(ListPos p);	// Sposta alla posizione p

  ListPos GoToFirst()		// Posiziona sul primo elemento della Lista
    { return(current = sentinel->next); }
  ListPos GoToLast()		// Posiziona sull'ultimo elemento della Lista
    { return(current = sentinel->prev); }
  ListPos GoToEndOfList()	// Posiziona sul terminatore di Lista
    { return(current = sentinel); }

  // Test vari
  int IsEmpty()	const		// La Lista e` vuota?
    { return(sentinel->next == sentinel); }
  int IsEndOfList() const	// Sono sulla sentinella?
    { return(current == sentinel); }
  int IsFirst() const		// Sono sul primo elemento?
    { return(current == sentinel->next); }

  // Inserimento e rimozione dati
  ListPos Insert(const mytype &d);	// Aggiunge in corrente nella Lista
  ListPos InsertFirst(const mytype &d);	// Aggiunge in testa alla Lista
  ListPos InsertLast(const mytype &d);	// Aggiunge in coda alla Lista

  mytype Delete();		// Rimuove l'elemento corrente dalla Lista (e
				// lo restituisce)

  ListPos Write(const mytype &d);// Scrive un nuovo dato nell'elemento corrente
  mytype Read() const;		// Restituisce l'elemento nella posizione
				// corrente della Lista


  // Varie funzioni
  ListPos Clear();		// Svuota la lista senza distruggerla
  ListPos Current()		// Ritorna la posizione corrente
    { return(current); }

  int Size() const		// Ritorna la dimensione attuale della lista
    { return(len); }

};
#endif
