//
// Implementazione della Classe List: doppi puntatori con "sentinella"
//


#include <iostream.h>
#include "List.H"


// Costruttore per il tipo di dato Lista.
List::List()
{
  sentinel = new Node;				// Creo il nodo sentinella
  sentinel->next = sentinel->prev = sentinel;	// Inizializzo la sentinella

  current = sentinel;

  len = 0;
}


// Distruttore per il tipo di dato Lista
List::~List()
{
  GoToFirst();			// Vai al primo elemento della Lista

  while (!IsEndOfList())	// Finche non sono arrivato alla sentinella
    Delete();			// Cancella un elemento

  delete sentinel;		// Elimina infine anche la sentinella
}


// Svuota la lista senza distruggerla
ListPos List::Clear()
{
  GoToFirst();			// Vai al primo elemento della Lista

  while (!IsEndOfList())	// Finche non sono arrivato alla sentinella
    Delete();			// Cancella un elemento

  return(current);
}


// Restituisce l'elemento della Lista nella posizione corrente.
mytype List::Read() const
{
  if (IsEndOfList()) {
    cerr << "Error: Read(), you can't access the sentinel value!" << endl;
    exit(-2);
  }

  return(current->data);
}



// Spostiamo il nodo corrente sul nodo successivo
ListPos List::Next()
{
  if (IsEndOfList()) {		// Errore se siamo alla fine della Lista
    cerr << "Error: Next(), You can't go beyond the sentinel!"
	 << endl;
    exit(-2);
  }

  return(current = current->next);
}


// Spostiamo il nodo corrente sul nodo precedente
ListPos List::Previous()
{
  if (IsFirst()) {		// Non si puo` andare oltre il primo elemento
    cerr << "Error: Previous(), You can't jump on the sentinel!"
	 << endl;
    exit(-2);
  }

  return(current = current->prev);
}



// !!!!!!!!!!!!!!!!!!!!!!!!!!!

// QUALCHE CHECK DA FARE?

// Sposta alla posizione p
ListPos List::GoTo(ListPos p)
{
  return(current = p);
}


// Inserimento nella Posizione corrente
ListPos List::Insert(const mytype &d)
{
  Node *newel = new Node;	// Costruiamo un nuovo nodo

  newel->data = d;		// Inizializzo il valore del nodo
  newel->prev = current->prev;	// Il nuovo nodo e` inserito PRIMA del nodo
  newel->next = current;	// corrente

  current->prev = newel;
  newel->prev->next = newel;

  len++;

  return(newel);
}


// Inserimento nella prima posizione
ListPos List::InsertFirst(const mytype &d)
{
  GoToFirst();

  return(Insert(d));
}


// Inserimento nell'ultima posizione
ListPos List::InsertLast(const mytype &d)
{
  GoToEndOfList();

  return(Insert(d));
}


// Cancella l'elemento nella posizione corrente
mytype List::Delete()
{
  mytype tmp;

  if (IsEndOfList()) { 
    cerr << "Error: Delete(), You can't delete the sentinel!"
	 << endl;
    exit(-2);
  }

  Node *tmpp = current->next;
  tmp = current->data;

  current->next->prev = current->prev;
  current->prev->next = current->next;

  delete current;

  len--;

  current = tmpp;

  return(tmp);
}


// Cambia il campo informazione del nodo corrente
ListPos List::Write(const mytype &d)
{
  if (IsEndOfList()) {
    cerr << "Error: Write(), You can't write on the sentinel!" << endl;
    exit(-2);
  }

  current->data = d;

  return(current);
}









