#include <iostream.h>
#include "Graph.H"
#include "Buckets.H"

void usageSyntaxHelp(int argc, char **argv);

int check_0_1(int n, const int * & x) {
  for(int v = n; v; v--)
    if(x[v] && x[v] != 1) {
      cerr << "Error in function check_0_1 : x[" << v << "] = " << x[v] << endl;
      return 0;
    }
  return 1;
}

int check_balance(int n, const int * x) {
  int  size1 = 0;
  for(int v = n; v; v--)
    if(x[v]) ++size1;
  if(size1 != n/2)  {
    cerr << "Error in function check_balance: size1 = " << size1 << " <> " << n/2 << " = n/2" << endl;
    return 0;
  }
  return 1;
}

int check_cut_value(Graph * G, const int * x, const int & cut_value) {
  if(G->size_of_cut(x) != cut_value)  {
    cerr << "Error in function check_cut_value: cut_value = " << cut_value
         << " <> " << G->size_of_cut(x) << " = G->size_of_cut(x)" << endl;
    return 0;
  }
  return 1;
}

int legality_check(Graph * G, const int * x, const int cut_value) {
  if(check_0_1(G->n(), x))
    if(check_balance(G->n(), x))
      if(check_cut_value(G, x, cut_value))
        return 1;
  return 0;
}

void randomPermutation(int *v, int n)
    // random permutation of the naturals in [1, ... ,n]
    // placed in v[1], ... ,v[n]
{
  int original[n];
  
  for(int i = n; i; i--)  original[i-1] = i;
  while(n > 0)  {
    int extract = dado(n-1);  // 0, ... , n-1
    v[n] = original[extract];
    original[extract] = original[--n];
  }
}

void diffGready(Graph * G, int * x, int & cut_value, int randomSeed) {
  int * to1_minus_to0 = new int[G->n()+1];
  for(int v = G->n(); v; v--) {
    x[v] = -1;
    to1_minus_to0[v] = 0;
  }
      
  Buckets e1e0(G->n(), G->max_degree());   // edges to 1 minus edges to 0.
   
  int permutation[G->n()+1];
  randomPermutation(permutation, G->n());
   
  for(int index = G->n(); index; index--) {
    int v = permutation[index];
    e1e0.insertLast(v, to1_minus_to0[v]);
  }   
   
  int toaddto = 1;		// 0 or 1
  int best_node;
  int free_nodes = G->n();
  while( free_nodes )  {
    toaddto = (1 -toaddto);	// flip!
     
    if(toaddto == 0)    // minimize E(i,1) - E(i,0)
      best_node = e1e0.extract_last_min_node();
    else   // minimize  E(i,0) - E(i,1) .....i.e.... maximize E(i,1) - E(i,0)
      best_node = e1e0.extract_first_max_node();
      // gets first (randomization at the beginning, with random permutation)

    x[best_node] = toaddto;
    free_nodes--;	

    for(int nei = G->first_neighbor(best_node); nei; nei = G->next_neighbor()) {
      // if not yet inserted, move "nei" upward in e_bucket[toaddto]
      if(x[nei] == -1)  // (not yet removed from buckets)
        if(toaddto)  e1e0.upFirst(nei, to1_minus_to0[nei]);
        else  e1e0.downLast(nei, to1_minus_to0[nei]);
      if(toaddto)  to1_minus_to0[nei]++;
      else  to1_minus_to0[nei]--;
    }   // for all neighbors of best_node. 
  }   // END  while  some nodes are neither set to 0 nor to 1.

  cut_value = G->size_of_cut(x);
}




int main(int argc, char **argv) {
  usageSyntaxHelp(argc, argv);
   
  Graph* G = new Graph(DIMACS, argv[1]);
  int randomSeed = 777;

  int * x = new int[G->n()];   int cut_value = 0;

  diffGready(G, x, cut_value, randomSeed);

  if(! legality_check(G, x, cut_value) ) {
    cerr << "\n ***  Sorry, legality_check()  failed  ***\n" << endl;
    exit(1);
  }


  cout << "Ho trovato una soluzione di valore " << cut_value << endl;
  cout << "Ecco una delle due shores del taglio trovato: " << cut_value << endl;
  
  for(int v = G->n(); v; v--)
    if(x[v])
      cout << v << endl;

  delete G;
  exit(0); 
}


void usageSyntaxHelp(int argc, char **argv) {
   if(argc != 2)  {

      exit(1);
   }
}

